// =======================================================================
// === Common Functions
// === Written by Pierre SMEYERS
// =======================================================================
// === This Javascript Library contains general functions for the online
// === doc runtime infrastructure.
// =======================================================================

// =========================================
// === String methods
// =========================================
String.prototype.endsWith = function(s)
{
	if (typeof s != 'string')
		throw('IllegalArgumentException: String.endsWith() is expecting a string argument.');
	var start = this.length - s.length;
	if(start < 0) return false;
	return this.substr(start) == s;
}
String.prototype.startsWith = function(s)
{
	if (typeof s != 'string')
		throw('IllegalArgumentException: String.startsWith() is expecting a string argument.');
	if(this.length < s.length)
		return false;
	return this.substr(0, s.length) == s;
}
String.prototype.trim = function()
{
    return this.replace(/(?:^\s+|\s+$)/g, "");
}
// =========================================
// === Array methods
// =========================================
Array.prototype.indexOf = function(o, fromIndex)
{
	if(!fromIndex)
		fromIndex = 0;
	for(var i=fromIndex; i<this.length; i++)
	{
		if(this[i] == o)
			return i;
	}
	return -1;
}
Array.prototype.lastIndexOf = function(o, fromIndex)
{
	if(!fromIndex)
		fromIndex = this.length-1;
	for(var i=fromIndex; i>=0; i--)
	{
		if(this[i] == o)
			return i;
	}
	return -1;
}
Array.prototype.contains = function(o)
{
	return this.indexOf(o) >= 0;
}
// =========================================
// === Url Analysis Functions
// =========================================
/**
 * Extracts arguments from the given location object.
 * Arguments are passed in a URL with the grammar:
 * http://server/the/path/to/the/file?arg1=val1&arg2=val2&...
 */
function getUrlParameter(name)
{
	var loc = window.location;
	if(!loc.parameters)
	{
		// --- parse parameters
		loc.parameters = new Object();
		var varAndVals = loc.search.substr(1).split("&");
		for(var i=0; i<varAndVals.length; i++)
		{
			var v = varAndVals[i].split("=");
			loc.parameters[unescape(v[0])] = unescape(v[1]);
		}
	}
	return loc.parameters[name];
}
// =========================================
// === DOM Navigation Helpers
// =========================================
function getChildren(nodeName, node)
{
	var ret = [];
	for(var i=0; i<node.childNodes.length; i++)
	{
		if(node.childNodes[i].nodeName == nodeName)
			ret.push(node.childNodes[i])
	}
	return ret;
}
function getFirstChild(nodeName, node)
{
	for(var i=0; i<node.childNodes.length; i++)
	{
		if(node.childNodes[i].nodeName == nodeName)
			return node.childNodes[i];
	}
	return null;
}
/**
 * Returns the first father node with given tag name.
 * This function pops the DOM hirearchy and returns the parent node with the given tag name.
 */
function getFatherNode(nodeName, node)
{
	var obj = node;
	while(obj != null && obj.nodeName != nodeName)
		obj = obj.parentNode;
	return obj;
}
/**
 * Returns the first father node with the given tag name and a non-null ID.
 */
function getFatherNodeWithId(nodeName, node)
{
	var obj = node;
	while(obj != null)
	{
		if(obj.nodeName == nodeName && obj.id != null && obj.id.length == 0)
			return obj;
		obj = obj.parentNode;
	}
	return null;
}
function getDOMObjDesc(obj)
{
	var ret = "<"+obj.nodeName;
	if(obj.id)
		ret += " id='"+obj.id+"'";
	if(obj.className)
		ret += " class='"+obj.className+"'";
	ret += " ...>";
	return ret;
}
/**
 * Returns the postion of obj relatively to the given parent (optional).
 */
function getAbsPos(obj, relativeToAncestor)
{
	var pos = {top: 0, left: 0};
	if(relativeToAncestor == null)
		relativeToAncestor = obj.ownerDocument.body;
	// --- if browser support getBoundingClientRect(), use it
	if(obj.getBoundingClientRect)
	{
		var rect = obj.getBoundingClientRect();
		pos.top = rect.top;
		pos.left = rect.left;
		if(relativeToAncestor != obj.ownerDocument.body)
		{
			// --- relative to a non-Body element
			var relativeRect = relativeToAncestor.getBoundingClientRect();
			pos.left -= relativeRect.left;
			pos.top -= relativeRect.top;
		}
		else
		{
			// --- add scroll position and remove body borders
//			var viewport = document.all ? obj.ownerDocument.documentElement : relativeToAncestor;
			var viewport = obj.ownerDocument.documentElement;
			pos.left += viewport.scrollLeft - viewport.clientLeft;
			pos.top += viewport.scrollTop - viewport.clientTop;
		}
		return pos;
	}
	// --- otherwise, compute position
//var message = "";
	while(true)
	{
//message += "Position of "+getDOMObjDesc(obj)+": "+obj.offsetTop+","+obj.offsetLeft;
		// --- add offset position relative to the offset parent
		pos.top += obj.offsetTop;
		pos.left += obj.offsetLeft;
		// --- pop to the offset parent, and break if it's the relative object
		obj = obj.offsetParent;
		if(obj == null || obj == relativeToAncestor) break;
//message += " relative to "+getDOMObjDesc(obj)+" (client pos: "+obj.clientTop+","+obj.clientLeft+") (scroll pos: "+obj.scrollTop+","+obj.scrollLeft+")\n";
		// --- compensate offset parent borders
		pos.top += obj.clientTop;
		pos.left += obj.clientLeft;
		// --- compensate offset parent scroll position
		pos.top -= obj.scrollTop;
		pos.left -= obj.scrollLeft;
	}
//alert(message);
	return pos;
}
// =========================================
// === XHR Helpers
// =========================================
/**
 * This function loads the given xml file and returns its root node.
 */
/*
function loadXML(xmlUrl)
{
	if(document.all)
	{
		// --- Internet Explorer
//		var xmlNode = new ActiveXObject("Microsoft.XMLDOM");
		var xmlNode = new ActiveXObject("Msxml2.DOMDocument");
		xmlNode.async = false;
		xmlNode.load(xmlUrl);
		return xmlNode;
	}
	else
	{
		// --- Mozilla
		var req = new XMLHttpRequest();
		req.overrideMimeType("text/xml");
		req.open("GET", xmlUrl, false);
		req.send(null);
		return req.responseXML;
	}
}
*/
/**
 * Instantiates a new XmlHttpRequest object (whatever the platform)
 */
function newXHR()
{
	if (window.XMLHttpRequest)
	{
		// --- Mozilla based browsers
		return new XMLHttpRequest();
	}
	else if (window.ActiveXObject)
	{
		// --- Internet Explorer (2 possible XML engines)
		try {
			return new ActiveXObject("Msxml2.XMLHTTP");
		} catch (e) {}
		try {
			return new ActiveXObject("Microsoft.XMLHTTP");
		} catch (e) {}
	}
	// --- browser does not support XML Http Request
	return null;
}
// =========================================
// === ClassName Helpers
// =========================================
function getClassIndex(fullcn, cn)
{
	if(fullcn == null)
		return -1;
	var i = fullcn.indexOf(cn);
	while(i >= 0)
	{
		// --- check separators before and after cn
		var idxAfter = i+cn.length;
		if((i == 0 || fullcn.charAt(i-1) == ' ') // separator before
			&& (idxAfter >= fullcn.length || fullcn.charAt(idxAfter) == ' ')) // separator after
			return i;
		i=fullcn.indexOf(cn, idxAfter);
	}
	return -1;
}
function hasClass(elt, cn)
{
	return getClassIndex(elt.className, cn) >= 0;
}
function addClass(elt, cn)
{
	if(hasClass(elt, cn))
		return false;
	if(elt.className == null || elt.className.length == 0)
		elt.className = cn;
	else
		elt.className += " "+cn;
	return true;
}
function removeClass(elt, cn)
{
	var idx = getClassIndex(elt.className, cn);
	if(idx < 0)
		return false;
	if(idx == 0)
	{
		// --- cn is first class
		if(cn.length == elt.className.length)
			// --- cn is the only class: reset className
			elt.className = null;
		else
			// --- cut className
			elt.className = elt.className.substr(cn.length+1);
	}
	else
	{
		// --- there are classes before cn
		if(idx + cn.length == elt.className.length)
			// --- cn is last
			elt.className = elt.className.substr(0, idx-1);
		else
			// --- cn in the middle
			elt.className = elt.className.substr(0, idx)+elt.className.substr(idx+cn.length+1);
	}
	return true;
}